@@ -14,13 +14,14 @@ This file is part of Ojota.
     You should have received a copy of the GNU  Lesser General Public License
     along with Ojota.  If not, see <http://www.gnu.org/licenses/>.
 """
+from collections import MutableSequence
 from json import dumps
+from threading import current_thread
 
 import ojota.sources
 
 from ojota.sources import JSONSource
 from ojota.cache import Cache
-from threading import current_thread
 
 
 def current_data_code(data_code):
@@ -29,7 +30,8 @@ def current_data_code(data_code):
 
 
 def get_current_data_code():
-    return Ojota._data_codes[current_thread()]
+    data_code = Ojota._data_codes.get(current_thread(), "")
+    return data_code
 
 
 def set_data_source(data_path):
@@ -61,7 +63,7 @@ class Relation(object):
         def _inner(method_self):
             """Inner function to return the property for the relation."""
             fk = getattr(method_self, self.attr_fk)
-            return self.to_class.get(fk)
+            return self.to_class.one(fk)
 
         ret = property(_inner)
         return ret
@@ -74,7 +76,7 @@ class Relation(object):
             relation."""
             pk = method_self.primary_key
             params = {self.attr_fk: pk}
-            return from_class.all(**params)
+            return from_class.many(**params)
 
         if self.related_name:
             prop = property(_inner)
@@ -82,6 +84,22 @@ class Relation(object):
             self.to_class.backwards_relations.append(self.related_name)
 
 
+class Callback(object):
+    def __init__(self, field_name, function):
+        self.field_name = field_name
+        self.function = function
+
+    def get_property(self):
+        """Returns the property in which the relation will be referenced."""
+        def _inner(method_self):
+            """Inner function to return the property for the relation."""
+            fk = getattr(method_self, self.field_name)
+            return self.function(fk)
+
+        ret = property(_inner)
+        return ret
+
+
 class WSRelation(Relation):
     """Adds a relation to another object."""
     def __init__(self, attr_fk, to_class, related_name=None, ws_call=None,
@@ -115,12 +133,12 @@ class WSRelation(Relation):
                 plural_name = method_self.plural_name
             if not self.ws_call:
                 _klass.data_source.get_all_cmd = self.ws_call
-                ret = _klass.get(getattr(method_self, self.attr_fk))
+                ret = _klass.one(getattr(method_self, self.attr_fk))
             else:
                 _klass.__class__.plural_name = "/".join(
                     (plural_name, getattr(method_self, self.attr_fk)))
                 _klass.data_source.get_all_cmd = self.ws_call
-                ret = _klass.all()
+                ret = _klass.many()
             _klass.__class__.plural_name = self.__plural_name
             _klass.data_source.get_all_cmd = self.__get_all_cmd
             return ret
@@ -128,7 +146,7 @@ class WSRelation(Relation):
         def _inner(method_self):
             """Inner function to return the property for the relation."""
             fk = getattr(method_self, self.attr_fk)
-            return self.to_class.get(fk)
+            return self.to_class.one(fk)
 
         if self.ws_call is not None:
             ret = property(_ws_inner)
@@ -137,6 +155,52 @@ class WSRelation(Relation):
         return ret
 
 
+class OjotaSet(MutableSequence):
+    def __init__(self, ojota_class, data):
+        super(OjotaSet, self).__init__()
+        self._list = list(data)
+        self.ojota_class = ojota_class
+
+    def __len__(self):
+        return len(self._list)
+
+    def __getitem__(self, indexes):
+        if isinstance(indexes, slice):
+            list_ = self._list[indexes.start:indexes.stop:indexes.step]
+            ret = OjotaSet(self.ojota_class, list_)
+        else:
+            ret = self.ojota_class(**self._list[indexes])
+
+        return ret
+
+    def __delitem__(self, ii):
+        del self._list[ii]
+
+    def __setitem__(self, ii, val):
+        return self._list[ii]
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        return "OjotaSet containing %s %s" % (len(self._list),
+                                              self.ojota_class.plural_name)
+
+    def insert(self, ii, val):
+        self._list.insert(ii, val)
+
+    def append(self, val):
+        list_idx = len(self._list)
+        self.insert(list_idx, val)
+
+    def many(self, **kwargs):
+        elems = self.ojota_class._filter(self._list, kwargs)
+        return OjotaSet(self.ojota_class, elems)
+
+    def one(self, **kwargs):
+        return self.ojota_class.one(**kwargs)
+
+
 class MetaOjota(type):
     """Metaclass for Ojota"""
     def __init__(self, *args, **kwargs):
@@ -147,6 +211,9 @@ class MetaOjota(type):
                 value.set_reversed_property(self)
                 setattr(self, attr, value.get_property())
                 self.relations[value.attr_fk] = (value.to_class, attr, value)
+            elif isinstance(value, Callback):
+                setattr(self, attr, value.get_property())
+
         return super(MetaOjota, self).__init__(*args, **kwargs)
 
 
@@ -162,6 +229,7 @@ class Ojota(object):
     cache = Cache()
     data_source = JSONSource()
     default_order = None
+    queryset_type = OjotaSet
 
     @property
     def primary_key(self):
@@ -236,7 +304,7 @@ class Ojota(object):
     @classmethod
     def _objetize(cls, data):
         """Return the data into an element."""
-        return [cls(**element_data) for element_data in data]
+        return cls.queryset_type(cls, data)
 
     @classmethod
     def _test_expression(cls, expression, value, element_data):
@@ -339,11 +407,15 @@ class Ojota(object):
                 reverse = False
 
             data_list = sorted(data_list, key=lambda e:
-                               getattr(e, order_field), reverse=reverse)
+                               e.get(order_field), reverse=reverse)
         return data_list
 
     @classmethod
-    def all(cls, **kargs):
+    def all(cls):
+        return cls.many()
+
+    @classmethod
+    def many(cls, **kargs):
         """Returns all the elements that match the conditions."""
         elements = cls._read_all_from_datasource().values()
         order_fields = cls.default_order
@@ -354,15 +426,14 @@ class Ojota(object):
         if kargs:
             elements = cls._filter(elements, kargs)
 
-        list_ = cls._objetize(elements)
-
         if order_fields:
-            list_ = cls._sort(list_, order_fields)
+            elements = cls._sort(elements, order_fields)
 
+        list_ = cls._objetize(elements)
         return list_
 
     @classmethod
-    def get(cls, pk=None, **kargs):
+    def one(cls, pk=None, **kargs):
         """Returns the first element that matches the conditions."""
         element = None
         if pk is not None:
@@ -377,9 +448,9 @@ class Ojota(object):
                 if pk in all_elems:
                     element = cls(**all_elems[pk])
         else:
-            result = cls.all(**kargs)
+            result = cls.many(**kargs)
             if result:
-                return result[0]
+                element = result[0]
 
         return element
 
@@ -410,7 +481,7 @@ class Ojota(object):
 
     def dump_values(self, new_data=None, delete=False):
         """Saves the data into a file."""
-        elements = self.__class__.all()
+        elements = self.__class__.many()
         json_data = []
         for element in elements:
             if element == self:
@@ -445,11 +516,65 @@ class Ojota(object):
                 if attr_name not in ojota_fields:
                     self.fields.append(attr_name)
                     new_data[attr_name] = attr_value
-            if self.__class__.get(self.primary_key) is None:
+            if self.__class__.one(self.primary_key) is None:
                 self.dump_values(new_data)
             else:
                 self.update(**new_data)
 
     @classmethod
     def preload(cls):
-        cls.all()
+        cls.many()
+
+
+class OjotaHierarchy(Ojota):
+    @property
+    def segments(self):
+        return self.primary_key.split(".")
+
+    @property
+    def root(self):
+        return self.segments[0]
+
+    @property
+    def last_segment(self):
+        return self.segments[-1]
+
+    @property
+    def parent(self):
+        if len(self.segments) > 1:
+            return self.one('.'.join(self.segments[:-1]))
+        else:
+            return None
+
+    def is_parent(self, other):
+       parent_id = '.'.join(self.segments[:-1])
+       return parent_id == other.primary_key
+
+    def is_ancestor(self, other):
+        if self.primary_key.startswith(other.primary_key):
+            return True
+        else:
+            return False
+
+    def is_sibling(self, other):
+        return self.segments[:-1] == other.segments[:-1]
+
+    def siblings(self):
+        args = {"%s__startswith" % self.pk_field: self.parent.primary_key}
+        elements = self.many(**args)
+        list_ = []
+        for element in elements:
+            if element.is_parent(self.parent):
+                list_.append(element)
+
+        return list_
+
+    def children(self):
+        args = {"%s__startswith" % self.pk_field: self.primary_key}
+        elements = self.many(**args)
+        list_ = []
+        for element in elements:
+            if element.is_parent(self):
+                list_.append(element)
+
+        return list_
